#include "raycaster.hpp"
#include <iostream>
#include <chrono>
#include <numeric>
#include <functional>

#define RAY_VIEW_WIDTH 960
#define RAY_VIEW_HEIGHT 720
#define RAY_VIEW_X (1280 - RAY_VIEW_WIDTH)
#define RAY_VIEW_Y 0

static const int SCREEN_HEIGHT=720;
static const int SCREEN_WIDTH=1280;

Matrix MAP{
  {1,1,1,1,1,1,1,1,1},
  {1,0,2,0,0,0,0,0,1},
  {1,0,4,0,0,5,2,0,1},
  {1,0,0,0,0,0,0,0,1},
  {1,1,0,0,0,0,0,1,1},
  {1,0,0,1,3,4,0,0,1},
  {1,0,0,0,0,0,1,1,1},
  {1,0,0,0,0,0,0,0,1},
  {1,1,1,1,1,1,1,1,1}
};

int main() {
  using KB = sf::Keyboard;

  sf::RenderWindow window(sf::VideoMode(SCREEN_WIDTH, SCREEN_HEIGHT), "Zed's Ray Caster Game Thing");

  //ZED this should set with a function
  float player_x = matrix::width(MAP) / 2;
  float player_y = matrix::height(MAP) / 2;

  Raycaster rayview(window, MAP, RAY_VIEW_WIDTH, RAY_VIEW_HEIGHT);
  rayview.set_position(RAY_VIEW_X, RAY_VIEW_Y);
  rayview.position_camera(player_x, player_y);

  double moveSpeed = 0.1;
  double rotSpeed = 0.1;

  while(window.isOpen()) {
    rayview.render();

    // DRAW GUI
    window.display();

    if(KB::isKeyPressed(KB::W)) {
      rayview.run(moveSpeed, 1);
    } else if(KB::isKeyPressed(KB::S)) {
      rayview.run(moveSpeed, -1);
    }

    if(KB::isKeyPressed(KB::D)) {
      rayview.rotate(rotSpeed, -1);
    } else if(KB::isKeyPressed(KB::A)) {
      rayview.rotate(rotSpeed, 1);
    }

    sf::Event event;
    while(window.pollEvent(event)) {
      if(event.type == sf::Event::Closed) {
        window.close();
      }
    }
  }

  return 0;
}
